# src/file_conversor/backend/gui/_components/padding.py

from re import M
from file_conversor.utils.dominate_utils import *


class PaddingAll:
    P_AUTO = "p-auto"
    P_0 = "p-0"
    P_1 = "p-1"
    P_2 = "p-2"
    P_3 = "p-3"
    P_4 = "p-4"
    P_5 = "p-5"
    P_6 = "p-6"


class PaddingX:
    P_AUTO = "px-auto"
    P_0 = "px-0"
    P_1 = "px-1"
    P_2 = "px-2"
    P_3 = "px-3"
    P_4 = "px-4"
    P_5 = "px-5"
    P_6 = "px-6"


class PaddingY:
    P_AUTO = "py-auto"
    P_0 = "py-0"
    P_1 = "py-1"
    P_2 = "py-2"
    P_3 = "py-3"
    P_4 = "py-4"
    P_5 = "py-5"
    P_6 = "py-6"


class PaddingLeft:
    P_AUTO = "pl-auto"
    P_0 = "pl-0"
    P_1 = "pl-1"
    P_2 = "pl-2"
    P_3 = "pl-3"
    P_4 = "pl-4"
    P_5 = "pl-5"
    P_6 = "pl-6"


class PaddingRight:
    P_AUTO = "pr-auto"
    P_0 = "pr-0"
    P_1 = "pr-1"
    P_2 = "pr-2"
    P_3 = "pr-3"
    P_4 = "pr-4"
    P_5 = "pr-5"
    P_6 = "pr-6"


class PaddingTop:
    P_AUTO = "pt-auto"
    P_0 = "pt-0"
    P_1 = "pt-1"
    P_2 = "pt-2"
    P_3 = "pt-3"
    P_4 = "pt-4"
    P_5 = "pt-5"
    P_6 = "pt-6"


class PaddingBottom:
    P_AUTO = "pb-auto"
    P_0 = "pb-0"
    P_1 = "pb-1"
    P_2 = "pb-2"
    P_3 = "pb-3"
    P_4 = "pb-4"
    P_5 = "pb-5"
    P_6 = "pb-6"


__all__ = [
    'PaddingAll',
    'PaddingX',
    'PaddingY',
    'PaddingTop',
    'PaddingBottom',
    'PaddingLeft',
    'PaddingRight',
]



# src\file_conversor\cli\pdf\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

SECURITY_PANEL = _(f"Security commands")
TRANSFORMATION_PANEL = _("Transformations")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "pdf"

# SUBCOMMANDS
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"

DECRYPT_NAME = "decrypt"
ENCRYPT_NAME = "encrypt"

EXTRACT_NAME = "extract"
EXTRACT_IMG_NAME = "extract-img"

MERGE_NAME = "merge"
OCR_NAME = "ocr"
REPAIR_NAME = "repair"

ROTATE_NAME = "rotate"
SPLIT_NAME = "split"



# src/file_conversor/backend/gui/config/_tab_audio_video.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.audio_video import FFmpegBackend

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation, AVAILABLE_LANGUAGES

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabConfigAudioVideo() -> tuple | list:
    return (
        FormFieldHorizontal(
            FormFieldSelect(
                *[
                    (f, f.upper())
                    for f in filter(lambda x: x != 'null', FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)
                ],
                current_value=CONFIG['video-format'],
                _name="video-format",
                help=_("Select the desired video format for video conversions."),
            ),
            label_text=_("Video Format"),
        ),
        VideoEncodingSpeedField(),
        VideoQualityField(),
        AudioBitrateField(),
        VideoBitrateField(),
    )


__all__ = ['TabConfigAudioVideo']



# src/file_conversor/utils/backend/pillow_parser.py

from pathlib import Path
from typing import Any

from rich import print
from rich.text import Text
from rich.panel import Panel
from rich.console import Group

# user-provided imports
from file_conversor.backend.image import PillowBackend

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_utils import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


class _PillowExifInfo:
    def __init__(self, input_file: Path, metadata: PillowBackend.Exif) -> None:
        super().__init__()
        self.input_file = input_file
        self.metadata = metadata

    def _parse(self):
        input_name = self.input_file.name
        in_ext = self.input_file.suffix.upper()[1:]
        return (input_name, in_ext)

    def rich(self):
        input_name, in_ext = self._parse()
        formatted = [
            f"{_('File Information')}:",
            f"  - {_('Name')}: {input_name}",
            f"  - {_('Format')}: {in_ext}",
        ]
        for tag, value in self.metadata.items():
            tag_name = PillowBackend.Exif_TAGS.get(tag, f"{tag}")
            formatted.append(f"  - {tag_name}: {value}")
        return formatted

    def div(self):
        input_name, in_ext = self._parse()
        with div() as result:
            div(f"{_('File Information')}:")
            div(f"  - {_('Name')}: {input_name}")
            div(f"  - {_('Format')}: {in_ext}")
            for tag, value in self.metadata.items():
                tag_name = PillowBackend.Exif_TAGS.get(tag, f"{tag}")
                div(f"  - {tag_name}: {value}")
            br()
        return result


class PillowParser:
    def __init__(self, backend: PillowBackend, input_file: Path) -> None:
        super().__init__()
        self.input_file = input_file
        self.backend = backend

    def run(self):
        try:
            self.metadata = self.backend.info(self.input_file)
        except Exception as e:
            raise RuntimeError(f"{_('File')} '{self.input_file}' {_('is corrupted or has inconsistencies')} - {repr(e)}")

    def get_exif_info(self):
        return _PillowExifInfo(self.input_file, self.metadata)


__all__ = [
    "PillowParser",
]



# src\file_conversor\backend\gui\image\_dom_page.py

from flask import url_for
from typing import Any

# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def image_index_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Image"),
        'url': url_for('image_index'),
        'active': active,
    }


def image_antialias_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Antialias"),
        'url': url_for('image_antialias'),
        'active': active,
    }


def image_blur_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Blur"),
        'url': url_for('image_blur'),
        'active': active,
    }


def image_compress_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Compress"),
        'url': url_for('image_compress'),
        'active': active,
    }


def image_convert_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Convert"),
        'url': url_for('image_convert'),
        'active': active,
    }


def image_enhance_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Enhance"),
        'url': url_for('image_enhance'),
        'active': active,
    }


def image_filter_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Filter"),
        'url': url_for('image_filter'),
        'active': active,
    }


def image_info_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Info"),
        'url': url_for('image_info'),
        'active': active,
    }


def image_mirror_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Mirror"),
        'url': url_for('image_mirror'),
        'active': active,
    }


def image_render_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Render"),
        'url': url_for('image_render'),
        'active': active,
    }


def image_resize_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Resize"),
        'url': url_for('image_resize'),
        'active': active,
    }


def image_rotate_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Rotate"),
        'url': url_for('image_rotate'),
        'active': active,
    }


def image_to_pdf_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("To PDF"),
        'url': url_for('image_to_pdf'),
        'active': active,
    }


def image_unsharp_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Unsharp"),
        'url': url_for('image_unsharp'),
        'active': active,
    }


__all__ = [
    'image_index_nav_item',
    'image_antialias_nav_item',
    'image_blur_nav_item',
    'image_compress_nav_item',
    'image_convert_nav_item',
    'image_enhance_nav_item',
    'image_filter_nav_item',
    'image_info_nav_item',
    'image_mirror_nav_item',
    'image_render_nav_item',
    'image_resize_nav_item',
    'image_rotate_nav_item',
    'image_to_pdf_nav_item',
    'image_unsharp_nav_item',
]




# src\file_conversor\cli\multimedia\unsharp_cmd.py
import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend
from file_conversor.cli.image._typer import FILTER_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, UNSHARP_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption, RadiusOption
from file_conversor.utils.validators import check_is_bool_or_none, check_path_exists, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def execute_image_unsharp_cmd(
    input_files: List[Path],
    radius: int,
    strength: int,
    threshold: int,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.unsharp_mask(
            input_file=input_file,
            output_file=output_file,
            radius=radius,
            percent=strength,
            threshold=threshold,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_unsharpened")

    logger.info(f"{_('Image unsharp')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=UNSHARP_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Applies unsharp mask to an image file.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {UNSHARP_NAME} input_file.jpg -od D:/Downloads`

        - `file_conversor {COMMAND_NAME} {UNSHARP_NAME} input_file1.bmp -r 3`

        - `file_conversor {COMMAND_NAME} {UNSHARP_NAME} input_file.jpg -s 100 -t 15`        
    """)
def unsharp(
    input_files: Annotated[List[Path], InputFilesArgument(PillowBackend)],

    radius: Annotated[int, RadiusOption()] = 2,

    strength: Annotated[int, typer.Option("--strenght", "-s",
                                          help=f'{_("Unsharp strength, in percent")}',
                                          min=1,
                                          )] = 130,

    threshold: Annotated[int, typer.Option("--threshold", "-t",
                                           help=f'{_("Threshold controls the minimum brightness change that will be sharpened")}',
                                           min=1,
                                           )] = 4,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_unsharp_cmd(
        input_files=input_files,
        radius=radius,
        strength=strength,
        threshold=threshold,
        output_dir=output_dir,
    )



# src\file_conversor\backend\gui\__init__.py

# user-provided modules
from file_conversor.backend.gui.web_app import WebApp
from file_conversor.backend.gui.flask_route import FlaskRoute

# routes
from file_conversor.backend.gui._api import routes as api_routes

from file_conversor.backend.gui.audio import routes as audio_routes
from file_conversor.backend.gui.config import routes as config_routes
from file_conversor.backend.gui.doc import routes as doc_routes
from file_conversor.backend.gui.ebook import routes as ebook_routes
from file_conversor.backend.gui.hash import routes as hash_routes
from file_conversor.backend.gui.image import routes as image_routes
from file_conversor.backend.gui.pdf import routes as pdf_routes
from file_conversor.backend.gui.ppt import routes as ppt_routes
from file_conversor.backend.gui.text import routes as text_routes
from file_conversor.backend.gui.video import routes as video_routes
from file_conversor.backend.gui.xls import routes as xls_routes

from file_conversor.backend.gui.icons import routes as icons_routes
from file_conversor.backend.gui.index import routes as index_routes


from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()

fapp = WebApp.get_instance()


# add context processors
fapp.add_context_processor(lambda: {
    'app': {
        'title': 'File Conversor',
        'version': Environment.get_version(),
    },
    '_base_status_bar': {
        'modal': {
            'title': _('Status Error'),
            'error': _('An error occurred while fetching the status'),
            'lost_conn': {
                'title': _('Connection Lost'),
                'msg': _('Lost connection to the server. Please check your network connection and try again.'),
            },
        },
        'processing': _('Processing...'),
        'failed': _('Failed'),
        'lost_conn': _('Connection Lost'),
        'completed': _('Completed'),
    },
    '_base_navbar': {
        'shutdown': {
            'btn_label': _('Shutdown'),
            'title': _('Server Shutdown'),
            'success': _('Server has been shut down successfully. Restart the app to use it again.'),
            'error': _('Unable to shut down the server'),
        },
    }
})

# api
fapp.add_route(api_routes())

# office
fapp.add_route(doc_routes())
fapp.add_route(xls_routes())
fapp.add_route(ppt_routes())

# other files
fapp.add_route(audio_routes())
fapp.add_route(video_routes())
fapp.add_route(image_routes())
fapp.add_route(pdf_routes())
fapp.add_route(ebook_routes())
fapp.add_route(text_routes())
fapp.add_route(hash_routes())

# UTILS CONFIG
fapp.add_route(config_routes())

fapp.add_route(icons_routes())
fapp.add_route(index_routes())

# export
__all__ = ["WebApp"]




# src\file_conversor\cli\video\rotate_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd import ffmpeg_cli_cmd

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, ROTATE_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_positive_integer, check_valid_options
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption, VideoBitrateOption, VideoEncodingSpeedOption, VideoQualityOption, VideoRotationOption

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="rotate_anticlock_90",
                description="Rotate Left",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{ROTATE_NAME}" "%1" -r -90',
                icon=str(icons_folder_path / "rotate_left.ico"),
            ),
            WinContextCommand(
                name="rotate_clock_90",
                description="Rotate Right",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{ROTATE_NAME}" "%1" -r 90',
                icon=str(icons_folder_path / "rotate_right.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=ROTATE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Rotate a video file (clockwise or anti-clockwise).')}

        {_('Outputs an video file with _rotated at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {ROTATE_NAME} input_file.webm -r 90 -od output_dir/ --audio-bitrate 192`

        - `file_conversor {COMMAND_NAME} {ROTATE_NAME} input_file.mp4 -r 180`
    """)
def rotate(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    rotation: Annotated[int, VideoRotationOption()],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)] = CONFIG["video-format"],

    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    ffmpeg_cli_cmd(
        input_files,
        file_format=file_format,
        out_stem="_rotated",
        rotation=rotation,
        video_bitrate=video_bitrate,
        video_encoding_speed=video_encoding_speed,
        video_quality=video_quality,
        output_dir=output_dir,
    )




# src\file_conversor\cli\ebook\convert_cmd.py

import typer

from rich import print

from typing import Annotated, Any, Callable, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import CalibreBackend

from file_conversor.cli.ebook._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = CalibreBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in CalibreBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name=f"to_{ext}",
                description=f"To {ext.upper()}",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "{ext}"',
                icon=str(icons_folder_path / f"{ext}.ico"),
            )
            for ext in CalibreBackend.SUPPORTED_OUT_FORMATS
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_ebook_convert_cmd(
    input_files: List[Path],
    format: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
) -> None:
    """Execute ebook conversion command."""
    calibre_backend = CalibreBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        calibre_backend.convert(
            input_file=input_file,
            output_file=output_file,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{format}")

    logger.info(f"{_('File conversion')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=CONVERT_NAME,
    help=f"""
        {_('Convert an ebook file to another ebook format (or PDF).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.epub -f pdf -od output_dir/`
        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.mobi -f epub`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(CalibreBackend.SUPPORTED_IN_FORMATS)],

    format: Annotated[str, FormatOption(CalibreBackend.SUPPORTED_OUT_FORMATS)],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_ebook_convert_cmd(
        input_files=input_files,
        format=format,
        output_dir=output_dir,
    )



# src/file_conversor/backend/gui/_api/doc/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.doc.convert import api_doc_convert


def routes():
    return [
        FlaskRoute(
            rule="/api/doc/convert",
            handler=api_doc_convert,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']




# src\file_conversor\cli\video\convert_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd import ffmpeg_cli_cmd

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import (InputFilesArgument, FormatOption, OutputDirOption,
                                              AudioBitrateOption, VideoBitrateOption,
                                              AudioCodecOption, VideoCodecOption,
                                              AxisOption,
                                              BrightnessOption, ColorOption, ContrastOption, GammaOption,
                                              VideoRotationOption, DeshakeOption, UnsharpOption,
                                              FPSOption, ResolutionOption,
                                              VideoEncodingSpeedOption, VideoQualityOption)

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_mkv",
                description="To MKV",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f mkv "%1" ',
                icon=str(icons_folder_path / 'mkv.ico'),
            ),
            WinContextCommand(
                name="to_mp4",
                description="To MP4",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f mp4 "%1"',
                icon=str(icons_folder_path / 'mp4.ico'),
            ),
            WinContextCommand(
                name="to_webm",
                description="To WEBM",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f webm "%1"',
                icon=str(icons_folder_path / 'webm.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=CONVERT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Convert a video file to another video format.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.webm -f mkv -od output_dir/ --audio-bitrate 192`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.mp4 -f avi -r 90`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.avi -f mp4 -rs 1280:720`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],
    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    audio_codec: Annotated[str | None, AudioCodecOption(FFmpegBackend.get_supported_audio_codecs())] = None,
    video_codec: Annotated[str | None, VideoCodecOption(FFmpegBackend.get_supported_video_codecs())] = None,

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    resolution: Annotated[str | None, ResolutionOption()] = None,
    fps: Annotated[int | None, FPSOption()] = None,

    brightness: Annotated[float, BrightnessOption()] = 1.0,
    contrast: Annotated[float, ContrastOption()] = 1.0,
    color: Annotated[float, ColorOption()] = 1.0,
    gamma: Annotated[float, GammaOption()] = 1.0,

    rotation: Annotated[int | None, VideoRotationOption()] = None,
    mirror_axis: Annotated[str | None, AxisOption()] = None,
    deshake: Annotated[bool, DeshakeOption()] = False,
    unsharp: Annotated[bool, UnsharpOption()] = False,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    ffmpeg_cli_cmd(
        input_files,
        file_format=file_format,
        audio_bitrate=audio_bitrate,
        video_bitrate=video_bitrate,
        audio_codec=audio_codec,
        video_codec=video_codec,
        video_encoding_speed=video_encoding_speed,
        video_quality=video_quality,
        resolution=resolution,
        fps=fps,
        brightness=brightness,
        contrast=contrast,
        color=color,
        gamma=gamma,
        rotation=rotation,
        mirror_axis=mirror_axis,
        deshake=deshake,
        unsharp=unsharp,
        output_dir=output_dir,
    )




# src\file_conversor\cli\pdf\convert_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.office import DOC_BACKEND
from file_conversor.backend.pdf import PyMuPDFBackend

from file_conversor.cli.pdf._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.cli.doc.convert_cmd import execute_doc_convert_cmd

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.formatters import format_in_out_files_tuple
from file_conversor.utils.typer_utils import DPIOption, FormatOption, InputFilesArgument, OutputDirOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = set([
    *PyMuPDFBackend.EXTERNAL_DEPENDENCIES,
    *DOC_BACKEND.EXTERNAL_DEPENDENCIES,
])


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in PyMuPDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_png",
                description="To PNG",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "png"',
                icon=str(icons_folder_path / 'png.ico'),
            ),
            WinContextCommand(
                name="to_jpg",
                description="To JPG",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "jpg"',
                icon=str(icons_folder_path / 'jpg.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_convert_cmd(
    input_files: List[Path],
    format: str,
    dpi: int,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
) -> None:
    if format in DOC_BACKEND.SUPPORTED_OUT_FORMATS:
        execute_doc_convert_cmd(
            input_files=input_files,
            format=format,
            output_dir=output_dir,
            progress_callback=progress_callback,
        )
        return

    with ProgressManager(len(input_files)) as progress_mgr:
        files = format_in_out_files_tuple(
            input_files=input_files,
            output_dir=output_dir,
            format=format,
        )
        logger.info(f"[bold]{_('Converting files')}[/] ...")
        # Perform conversion
        backend = PyMuPDFBackend(verbose=STATE['verbose'])
        for input_file, output_file in files:
            backend.convert(
                input_file=input_file,
                output_file=output_file,
                dpi=dpi,
            )
            progress_callback(progress_mgr.complete_step())

    logger.info(f"{_('File convertion')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=CONVERT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Convert a PDF file to a different format (might require Microsoft Word / LibreOffice).')}
        
        {_('Outputs a file with the PDF page number at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.pdf -f jpg --dpi 200`

        - `file_conversor -oo {COMMAND_NAME} {CONVERT_NAME} input_file.pdf -f png`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.pdf -f docx`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(["pdf"])],
    format: Annotated[str, FormatOption({
        **PyMuPDFBackend.SUPPORTED_OUT_FORMATS,
        **DOC_BACKEND.SUPPORTED_OUT_FORMATS,
    })],
    dpi: Annotated[int, DPIOption()] = CONFIG["image-dpi"],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_convert_cmd(
        input_files=input_files,
        format=format,
        dpi=dpi,
        output_dir=output_dir,
    )
